/******************************************************************************/
/*	PDB Dreamer version 1.00					      */
/*      (A.G. de Brevern - EBGM - E0346 INSERM)				      */
/******************************************************************************/
/*	turn.h	(beta version 2003)				      	      */
/*	------------							      */
/*	definition of turns used					      */
/******************************************************************************/
/*     NB_XXXXX_TURN defined in lib_define.h				      */
/*     extern defined in lib_structure.h                   	              */
/******************************************************************************/

# include "basic_h.h"

/**********************************/
/* definition of beta-turn (n=4) */
/**********************************/
/* 01 -> 09 */
/************/


double beta_turn[NB_BETA_TURN-1][4]={{-60.0,-30.0,-90.0,0.0},\
		{-60.0,120.0,80.0,0.0},\
		{-60.0,-30.0,-120.0,120.0},\
		{60.0,30.0,90.0,0.0},\
		{60.0,-120.0,-80.0,0.0},\
		{-60.0,120.0,-90.0,0.0},\
		{-120.0,120.0,-60.0,0.0},\
		{-135.0,135.0,-75.0,160.0}};

char *beta_turn_name[NB_BETA_TURN]={"Beta turn I         ",\
		"Beta turn II        ",\
		"Beta turn VIII      ",\
		"Beta inverse turn I ",\
		"Beta inverse turn II",\
		"Beta turn VIa1      ",\
		"Beta turn VIa2      ",\
		"Beta turn VIb       ",\
		"Beta Type IV        "};

char *beta_turn_ref={"Hutchinson et Thornton (1996)"};

char *beta_turn_note={"(1) +/- 30  for the angles and +/- 45 for one,\n(2) for VIa1, VIa2, VIb, the i+2 residue must be a cis-Proline"};


/**********************************/
/* definition of delta-turn (n=3) */
/**********************************/
/* 10 -> 11 */
/************/

double gamma_turn[NB_GAMMA_TURN][2]={{75.0,-64.0},\
		{-79.0,69.0}};

char *gamma_turn_name[NB_GAMMA_TURN]={"Gamma Classic",\
		"Gamma Inverse"};

char *gamma_turn_ref={"Rose et al. (1985)"};

char *gamma_turn_note={"(1) +/- 40  for the angles"};

/**********************************/
/* definition of alpha-turn (n=5) */
/**********************************/
/* 12 -> 21 */
/************/
	/***********************************/
	/* taken from Pavone et al. (1996) */
	/***********************************/
	/* 	0 : Type  I - alpha RS 	   */
	/* 	1 : Type  I - alpha LS     */
	/* 	2 : Type II - alpha RS     */
	/* 	3 : Type II - alpha LS     */
	/* 	4 : Type  I - alpha RU     */
	/* 	5 : Type  I - alpha LU     */
	/* 	6 : Type II - alpha RU     */
	/* 	7 : Type II - alpha LU     */
	/* 	8 : Type  I - alpha C      */
	/***********************************/
	/* 	9 : Type  I - alpha C'     */
	/* ADB	    no control over omega  */
	/***********************************/
	

double alpha_turn[NB_ALPHA_TURN][6]={{-60.0,-29.0,-72.0,-29.0,-96.0,-20.0},\
		{48.0,42.0,67.0,33.0,70.0,32.0},\
		{-59.0,129.0,88.0,-16.0,-91.0,-32.0},\
		{53.0,-137.0,-95.0,81.0,57.0,38.0},\
		{59.0,-157.0,-67.0,-29.0,-68.0,-39.0},\
		{-61.0,158.0,64.0,37.0,62.0,39.0},\
		{54.0,39.0,67.0,-5.0,-125.0,-34.0},\
		{-65.0,-20.0,-90.0,16.0,86.0,37.0},\
		{-103.0,143.0,-85.0,2.0,-54.0,-39.0},\
		{-103.0,143.0,-85.0,2.0,-54.0,-39.0}};

char *alpha_turn_name[NB_ALPHA_TURN]={"Type  I - alpha RS",\
		"Type  I - alpha LS",\
		"Type II - alpha RS",\
		"Type II - alpha LS",\
		"Type  I - alpha RU",\
		"Type  I - alpha LU",\
		"Type II - alpha RU",\
		"Type II - alpha LU",\
		"Type  I - alpha C'",\
		"Type  I - alpha C "};

char *alpha_turn_ref={"Pavone et al. (1996)"};

char *alpha_turn_note={"(1) The definition is for nonhelical alpha-turns,\n(2) Type  I - alpha C needs Omega = 0 for (i+1),\n(3) Type  I - alpha C' does not (ADB),\n(4) +/- 30  for the angles and +/- 45 for one"};


/*******************************/
/* definition of turns (Linus) */
/*******************************/
/*
double SriniRose_turn[NB_SRIROSE_TURN][6]={{-60.0,-30.0,-90.0,0.0,90.0,0.0},\
		{-60.0,-30.0,-90.0,0.0,-90.0,120.0},\
		{-90.0,120.0,-60.0,-30.0,-90.0,0.0},\
		{60.0,30.0,90.0,0.0,-90.0,120.0},\
		{-60.0,120.0,80.0,0.0,-90.0,120.0},\
		{-90.0,120.0,-60.0,120.0,80.0,0.0},\
		{60.0,-120.0,-80.0,0.0,-90.0,120.0,},\
		{-80.0,80.0,80.0,-80.0,-90.0,120.0},\
		{-100.0,-150.0,110.0,-160.0,-90.0,140.0},\
		{-120.0120.0,-70.0,150.0,-90.0,120.0}};

char *srinirose_turn_name[NB_ALPHA_TURN]={"Type  I - alpha RS",\
		"Type  I - alpha LS",\
		"Type II - alpha RS",\
		"Type II - alpha LS",\
		"Type  I - alpha RU",\
		"Type  I - alpha LU",\
		"Type II - alpha RU",\
		"Type II - alpha LU",\
		"Type  I - alpha C'",\
		"Type  I - alpha C "};

char *srinirose_turn_ref={"Pavone et al. (1996)"};

char *srinirose_turn_note={"(1) The definition is for nonhelical alpha-turns,\n(2) Type  I - alpha C needs Omega = 0 for (i+1),\n(3) Type  I - alpha C' does not (ADB),\n(4) +/- 30  for the angles and +/- 45 for one"};

*/
