/******************************************************************************/
/*	PDB Dreamer version x.yz					      */
/*      (A.G. de Brevern - EBGM - E0346 INSERM)				      */
/******************************************************************************/
/*	superposition.c	(beta version 2003)				      */
/*	---------------							      */
/*	core of superposition methods					      */
/******************************************************************************/

# include "../HEADER/basic_h.h"

# include "../HEADER/lib_define.h"				
# include "../HEADER/lib_protoC.h"
# include "../HEADER/options.h"

/***************************************************************/
/* computation of crms : coordinate root mean square deviation */
/***************************************************************/

double rmsd(double *X1,double *Y1,double *Z1,double *X2,double *Y2,double *Z2, long taille){
	long i;
	double somme_diff=0.0;
	
	for(i=0;i<taille;i++)
		somme_diff+=distance_intercarbone(X1[i],Y1[i],Z1[i],X2[i],Y2[i],Z2[i]);
		somme_diff/=((double)taille);
		somme_diff=sqrt(somme_diff);
	return(somme_diff);
}

double rmsd3d(double **A,double **B, long taille){
	long i;
	double somme_diff=0.0;
	
	for(i=0;i<taille;i++)
		somme_diff+=distance_intercarbone(A[i][0],A[i][1],A[i][2],B[i][0],B[i][1],B[i][2]);
		somme_diff/=((double)taille);
		somme_diff=sqrt(somme_diff);
	return(somme_diff);
}

double superposit(double *Xfixe,double *Yfixe,double *Zfixe,double *Xmobile,double *Ymobile,double *Zmobile, long taill_frag, long DEF_NBTRY, long WAIT_NEXT,long NB_MIN_GOOD){

	int i;
	
	int test=0;
	int nbtours=0;
	
	double alpha;
	double alpha_Zero=20.;
	
	double somme_diff=0.;
	double mini_somm=100000.;
	double meilleur_somme_diff=100000.;
	
	int lastgood=0;
	int good=0;
	int nbtry=0;
	
	double *Xmobileb,*Xfixeb,*Ymobileb,*Yfixeb,*Zmobileb,*Zfixeb;
	double *TheBestXmobileb,*TheBestYmobileb,*TheBestZmobileb;
	double *minXmobileb,*minYmobileb,*minZmobileb;
	
	if(DEF_NBTRY<0) DEF_NBTRY=300;					/*   nombre d'essai indpendant correct     = 300 */
	if(WAIT_NEXT<0) WAIT_NEXT=500;					/*  nombre d'attente jusqu'a arret  correct = 500 */
	if(NB_MIN_GOOD<0) NB_MIN_GOOD=15;				/*  nombre d'attente jusq'a reprise correct =  15 */
	
	srand48(time(NULL));
	
	/*if(Werboze=='T'){
		printf("rmsd (sa) step 00 (mol.1)\n");
		for(i=0;i<taill_frag;i++)printf("%3d\t%8.3lf\t%8.3lf\t%8.3lf\n",i,Xfixe[i],Yfixe[i],Zfixe[i]);
		printf("rmsd (sa) step 00 (mol.2)\n");
		for(i=0;i<taill_frag;i++)printf("%3d\t%8.3lf\t%8.3lf\t%8.3lf\n",i,Xmobile[i],Ymobile[i],Zmobile[i]);
	}*/
	
	Xfixeb=(double*)calloc(taill_frag,sizeof(double));
	Yfixeb=(double*)calloc(taill_frag,sizeof(double));
	Zfixeb=(double*)calloc(taill_frag,sizeof(double));
	
	Xmobileb=(double*)calloc(taill_frag,sizeof(double));
	Ymobileb=(double*)calloc(taill_frag,sizeof(double));
	Zmobileb=(double*)calloc(taill_frag,sizeof(double));
	
	TheBestXmobileb=(double*)calloc(taill_frag,sizeof(double));
	TheBestYmobileb=(double*)calloc(taill_frag,sizeof(double));
	TheBestZmobileb=(double*)calloc(taill_frag,sizeof(double));
	
	minXmobileb=(double*)calloc(taill_frag,sizeof(double));
	minYmobileb=(double*)calloc(taill_frag,sizeof(double));
	minZmobileb=(double*)calloc(taill_frag,sizeof(double));
	
			
while (nbtry<DEF_NBTRY){
	lastgood=good=0;
	test=0;
				/** 1- center **/
	
	for(i=0;i<taill_frag;i++){Xmobileb[i]=Xmobile[i]-Xmobile[(taill_frag/2)];Ymobileb[i]=Ymobile[i]-Ymobile[(taill_frag/2)];Zmobileb[i]=Zmobile[i]-Zmobile[(taill_frag/2)];}	
	for(i=0;i<taill_frag;i++){Xfixeb[i]=Xfixe[i]-Xfixe[(taill_frag/2)];Yfixeb[i]=Yfixe[i]-Yfixe[(taill_frag/2)];Zfixeb[i]=Zfixe[i]-Zfixe[(taill_frag/2)];}
	
	for(i=0;i<taill_frag;i++){Xfixeb[i]=Xfixe[i]-Xmobile[(taill_frag/2)];Yfixeb[i]=Yfixe[i]-Ymobile[(taill_frag/2)];Zfixeb[i]=Zfixe[i]-Zmobile[(taill_frag/2)];}		
	
		
	somme_diff=rmsd(Xmobileb,Ymobileb,Zmobileb,Xfixeb,Yfixeb,Zfixeb,taill_frag);
	
	/*if(Werboze=='T'){
		printf("rmsd (sa) step center %4d (mol.1)\n",nbtry);
		for(i=0;i<taill_frag;i++)printf("%3d\t%8.3lf\t%8.3lf\t%8.3lf\n",i,Xfixeb[i],Yfixeb[i],Zfixeb[i]);
		printf("rmsd (sa) step center %4d (mol.1)\n",nbtry);
		for(i=0;i<taill_frag;i++)printf("%3d\t%8.3lf\t%8.3lf\t%8.3lf\n",i,Xmobileb[i],Ymobileb[i],Zmobileb[i]);
		printf("rmsd is %8.3lf [Sippl : %8.3lf] step center %4d (mol.1)\n",somme_diff,rmsd(Xmobile,Ymobile,Zmobile,Xfixe,Yfixe,Zfixe,taill_frag),nbtry);
	}*/
		
	mini_somm=somme_diff;

	for(i=0;i<taill_frag;i++){minXmobileb[i]=Xmobileb[i];minYmobileb[i]=Ymobileb[i];minZmobileb[i]=Zmobileb[i];}
			
	alpha=alpha_Zero;
	
	while(test==0){
		nbtours++;
		alpha=alpha_Zero/1+(1/nbtours);
		
		Rotat2axes(alpha,Xmobileb,Ymobileb);
		Rotat2axes(alpha,Ymobileb,Zmobileb);
		Rotat2axes(alpha,Zmobileb,Xmobileb);
		TranslatExt(alpha,Xmobileb,taill_frag);
		
		somme_diff=rmsd(Xmobileb,Ymobileb,Zmobileb,Xfixeb,Yfixeb,Zfixeb,taill_frag);	
	
		if(somme_diff<mini_somm){
			mini_somm=somme_diff;
			for(i=0;i<taill_frag;i++)
				{
				minXmobileb[i]=Xmobileb[i];
				minYmobileb[i]=Ymobileb[i];
				minZmobileb[i]=Zmobileb[i];
				}
			lastgood=good=-1;
			}
		
		if(good==NB_MIN_GOOD){
			for(i=0;i<taill_frag;i++)
				{
				Xmobileb[i]=minXmobileb[i];
				Ymobileb[i]=minYmobileb[i];
				Zmobileb[i]=minZmobileb[i];
				}
			good=-1;
			}	
		good++;	
		lastgood++;
		/*printf("%d %.2f %.2f %d %d\n",nbtours,somme_diff,mini_somm,lastgood,good);*/
		if(lastgood==WAIT_NEXT) test=1;
		}
		
		/*printf("%d %.2f %.2f %d %d\n",nbtours,somme_diff,mini_somm,lastgood,good);*/
		somme_diff=rmsd(minXmobileb,minYmobileb,minZmobileb,Xfixeb,Yfixeb,Zfixeb,taill_frag);
		
		if(somme_diff<meilleur_somme_diff){
			meilleur_somme_diff=somme_diff;
			for(i=0;i<taill_frag;i++){
				TheBestXmobileb[i]=minXmobileb[i];
				TheBestYmobileb[i]=minYmobileb[i];
				TheBestZmobileb[i]=minZmobileb[i];
				}
			lastgood=good=-1;
			}
		nbtry++;
		}			/*fini fin */
		
		
		somme_diff=rmsd(Xfixeb,Yfixeb,Zfixeb,TheBestXmobileb,TheBestYmobileb,TheBestZmobileb,taill_frag);
	
		/*for(i=0;i<taill_frag;i++){Xmobile[i]=TheBestXmobileb[i];Ymobile[i]=TheBestYmobileb[i];Zmobile[i]=TheBestZmobileb[i];}	
		for(i=0;i<taill_frag;i++){Xfixe[i]=Xfixeb[i];Yfixe[i]=Yfixeb[i];Zfixe[i]=Zfixeb[i];}
	*/
		free(Xmobileb);	free(Ymobileb);	free(Zmobileb);
		free(Xfixeb);	free(Yfixeb);	free(Zfixeb);
		free(TheBestXmobileb);	free(TheBestYmobileb);	free(TheBestZmobileb);
		free(minXmobileb);	free(minYmobileb);	free(minZmobileb);
		
		return(somme_diff);
}


void create_PDB_CA(double *X,double *Y,double *Z,char *name, char *file, long begin, long taill_frag){

	FILE *f0;
	int i;
	
	f0=fecrire(file);
	fprintf(f0,"HEADER\t\t\t\t\t%4s%c %6ld BP16  %1.1f\n",name,'A',taill_frag+1,1.0);
	for(i=0;i<taill_frag;i++){
			fprintf(f0,"ATOM    %3.0ld %3s %4s %c%4ld",i+begin,"CA ","GLY",'A',i+begin);
			fprintf(f0,"   %8.3f%8.3f%8.3f  1.00 19.19             %c\n",X[i],Y[i],Z[i],'C');
			}
	for(i=0;i<(taill_frag-1);i++)
	fprintf(f0,"CONECT %4ld %4ld\n",i+begin,i+begin+1);
	fprintf(f0,"END\n");
	fclose(f0);
}

void create_PDB_CAxyz(double **ValueXYZ,char *name, char *file, long begin, long taill_frag){

	FILE *f0;
	int i;
	
	f0=fecrire(file);
	fprintf(f0,"HEADER\t\t\t\t\t%4s%c %6ld BP16  %1.1f\n",name,'A',taill_frag+1,1.0);
	for(i=0;i<taill_frag;i++){
			fprintf(f0,"ATOM    %3.0ld %3s %4s %c%4ld",i+begin,"CA ","GLY",'A',i+begin);
			fprintf(f0,"   %8.3f%8.3f%8.3f  1.00 19.19             %c\n",ValueXYZ[i][0],ValueXYZ[i][1],ValueXYZ[i][2],'C');
			}
	for(i=0;i<(taill_frag-1);i++)
	fprintf(f0,"CONECT %4ld %4ld\n",i+begin,i+begin+1);
	fprintf(f0,"END\n");
	fclose(f0);
}
